package cn.fw.valhalla.controller.api;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.data.base.domain.common.Message;
import cn.fw.valhalla.domain.db.follow.ClueTask;
import cn.fw.valhalla.domain.dto.CustomerDetailDto;
import cn.fw.valhalla.sdk.api.ValhallaGeneralApiService;
import cn.fw.valhalla.sdk.param.*;
import cn.fw.valhalla.sdk.result.AccidentFollowerResult;
import cn.fw.valhalla.sdk.result.CustomerClueDeadline;
import cn.fw.valhalla.sdk.result.CustomerSimpleInfoDto;
import cn.fw.valhalla.sdk.result.InboundRateTargetResult;
import cn.fw.valhalla.service.bus.ReachLogBizService;
import cn.fw.valhalla.service.bus.cust.CustomerApiBizService;
import cn.fw.valhalla.service.bus.cust.CustomerBizService;
import cn.fw.valhalla.service.bus.follow.ClueApiBizService;
import cn.fw.valhalla.service.bus.follow.FollowApiBizService;
import cn.fw.valhalla.service.data.CustomerLoanInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import static cn.fw.common.businessvalidator.Validator.BV;
import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * @author : kurisu
 * @className : ValhallaGeneralApiServiceImpl
 * @description : 通用api实现
 * @date: 2020-11-11 16:01
 */
@Slf4j
@RestController
@RequestMapping("/api/valhalla/general")
public class ValhallaGeneralApiServiceImpl implements ValhallaGeneralApiService {

    private final ReachLogBizService reachLogBizService;
    /**
     * 保有客数据服务
     */
    private final CustomerBizService customerBiz;
    private final FollowApiBizService followApiBizService;
    private final ClueApiBizService clueApiBizService;
    private final CustomerApiBizService customerApiBizService;
    private final CustomerLoanInfoService customerLoanInfoService;

    @Autowired
    public ValhallaGeneralApiServiceImpl(final ReachLogBizService reachLogBizService,
                                         final CustomerBizService customerBiz,
                                         final FollowApiBizService followApiBizService,
                                         final ClueApiBizService clueApiBizService,
                                         final CustomerApiBizService customerApiBizService,
                                         final CustomerLoanInfoService customerLoanInfoService) {
        this.reachLogBizService = reachLogBizService;
        this.customerBiz = customerBiz;
        this.followApiBizService = followApiBizService;
        this.clueApiBizService = clueApiBizService;
        this.customerApiBizService = customerApiBizService;
        this.customerLoanInfoService = customerLoanInfoService;
    }

    @PostMapping("/reach/save")
    @Override
    @ControllerMethod("保存进站记录")
    public Message<Void> saveReachLog(@Valid @RequestBody ReachLogReq reachLogReq) {
        reachLogBizService.saveReachLog(reachLogReq);
        return success();
    }

    @Override
    @PostMapping("/query/customer/list")
    @ControllerMethod("查询保有客档案列表")
    public Message<List<CustomerSimpleInfoDto>> queryCustomList(@Valid @RequestBody CustomerQueryReq customerQueryReq) {
        List<CustomerDetailDto> list = customerBiz.queryCustomList(customerQueryReq);
        List<CustomerSimpleInfoDto> dtoList = new ArrayList<>();
        for (CustomerDetailDto customer : list) {
            CustomerSimpleInfoDto dto = new CustomerSimpleInfoDto();
            BeanUtils.copyProperties(customer, dto);
            dtoList.add(dto);
        }
        return success(dtoList);
    }


    /**
     * 查询车辆最短的一个跟进的截止日期（不包含事故车）
     *
     * @param vin
     * @return
     */
    @GetMapping("/query/customer/clue/deadline")
    @Override
    @ControllerMethod("查询车辆最短的一个跟进的截止日期")
    public Message<CustomerClueDeadline> queryClueDeadline(@RequestParam("vin") final String vin, @RequestParam("groupId") final Long groupId) {
        BV.isNotBlank(vin, () -> "vin不能为空");
        return success(clueApiBizService.queryClueDeadline(vin, groupId));
    }

    /**
     * 查询车辆最短的一个跟进的截止日期（不包含事故车）
     *
     * @param params
     * @return
     */
    @PostMapping("/query/customer/clue/batch/deadline")
    @Override
    @ControllerMethod("查询车辆最短的一个跟进的截止日期[批量]")
    public Message<List<CustomerClueDeadline>> queryClueDeadlineBatch(@Valid @RequestBody ClueDeadlineParams params) {
        return success(clueApiBizService.queryClueDeadlineBatch(params.getVinList(), params.getGroupId()));
    }


    @Override
    @GetMapping("/query/accident/follower")
    @ControllerMethod("查询进行中的事故车跟进人员")
    public Message<AccidentFollowerResult> queryByPlate(@RequestParam("plateNo") String plateNo, @RequestParam("groupId") Long groupId) {
        ClueTask clueTask = followApiBizService.queryAccidentFollowerByPlate(plateNo, groupId);
        if (Objects.isNull(clueTask)) {
            return success();
        }
        AccidentFollowerResult result = new AccidentFollowerResult();
        result.setUserId(clueTask.getFollowUser());
        result.setUserName(clueTask.getFollowUserName());
        result.setShopId(clueTask.getFollowShop());
        return success(result);
    }

    @Override
    @PostMapping("/query/inbound_rate_target")
    @ControllerMethod("查询进站率目标")
    public Message<InboundRateTargetResult> queryInboundRateTarget(@Valid @RequestBody final InboundRateTargetReq inboundRateTargetReq) {
        InboundRateTargetResult result = reachLogBizService.queryInboundRateTarget(inboundRateTargetReq);
        return success(result);
    }

    @Override
    @PostMapping("/distribution/customer")
    @ControllerMethod("分配保有客")
    public Message<Void> distribution(@Valid @RequestBody CustomerDistributionReq distributionReq) {
        customerApiBizService.distribution4api(distributionReq.getVin(), distributionReq.getStaffId(), distributionReq.getShopId());
        return success();
    }

    /**
     * <p>终止档案贷款期状态</p>
     * <p>提前还清贷款的情况</p>
     *
     * @param vin     车架号
     * @param groupId 集团id
     * @since 1.3.2
     */
    @Override
    @ControllerMethod("终止贷款期")
    @PostMapping("/stop/customer/loan")
    public Message<Void> stopCustomerLoan(@RequestParam("vin") String vin, @RequestParam("groupId") Long groupId) {
        customerLoanInfoService.stopLoan(vin, groupId);
        return success();
    }
}
