package cn.fw.valhalla.controller.debug

import cn.fw.common.web.annotation.ControllerMethod
import cn.fw.common.web.util.ResultBuilder.success
import cn.fw.data.base.domain.common.Message
import cn.fw.security.auth.client.annotation.Authorization
import cn.fw.security.auth.client.annotation.IgnoreAuth
import cn.fw.security.auth.client.enums.AuthType
import cn.fw.valhalla.common.utils.DateUtil
import cn.fw.valhalla.domain.db.report.CustomerRetentionRatio
import cn.fw.valhalla.domain.vo.RoleChangeUserVO
import cn.fw.valhalla.rpc.hermes.HermesRpcService
import cn.fw.valhalla.service.bus.cust.CommonBizService
import cn.fw.valhalla.service.bus.follow.FollowApiBizService
import cn.fw.valhalla.service.report.CustomerRetentionRatioBizService
import cn.fw.valhalla.service.report.PublicReportBizService
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.async
import kotlinx.coroutines.delay
import kotlinx.coroutines.future.future
import lombok.RequiredArgsConstructor
import org.slf4j.LoggerFactory
import org.springframework.validation.annotation.Validated
import org.springframework.web.bind.annotation.*
import java.util.*
import javax.validation.Valid
import javax.validation.constraints.NotBlank

/**
 * 调试接口
 *
 * @author : kurisu
 * @version : 1.0
 * @desc : 调试接口
 * @date : 2024-02-29 17:55
 */
@RestController
@RequiredArgsConstructor
@IgnoreAuth
@Authorization(AuthType.NONE)
@Validated
@RequestMapping("/debug")
class DebugController(
    private val publicReportBizService: PublicReportBizService,
    private val commonBizService: CommonBizService,
    private val hermesRpcService: HermesRpcService,
    private val customerRetentionRatioBizService: CustomerRetentionRatioBizService,
    private val followApiBizService: FollowApiBizService
) {
    private val log = LoggerFactory.getLogger(javaClass)

    @GetMapping("/run/publicReportData")
    @ControllerMethod("公共池报表数据准备")
    fun publicReportData(
        @RequestParam(required = false, defaultValue = "2") groupId: Long,
        @RequestParam start: Date,
        @RequestParam end: Date
    ): Message<Void> {
        val endTime = end.takeIf {
            it.after(Date())
        } ?: Date()

        if (start.after(endTime)) {
            throw RuntimeException("开始时间大于结束时间")
        }
        var dateStart = DateUtil.date2LocalDate(start)
        val dateEnd = DateUtil.date2LocalDate(endTime)
        log.info("入参dateStart: {}, dateEnd: {}", dateStart, dateEnd)
        if (dateStart.isBefore(dateEnd)) {
            while (dateStart.isBefore(dateEnd) || dateStart.isEqual(dateEnd)) {
                publicReportBizService.extracting(dateStart, groupId)
                dateStart = dateStart.plusDays(1)
            }
        } else {
            publicReportBizService.extracting(dateStart, groupId)
        }
        return success()
    }

    @IgnoreAuth
    @PostMapping("/cache/retention")
    @ControllerMethod("缓存角色变动人员的保持率数据")
    fun cacheRoleChangeUser(@RequestBody roleChangeUserVO: @Valid RoleChangeUserVO?): Message<Void> {
        customerRetentionRatioBizService.roleChangeUserStash(roleChangeUserVO)
        return success()
    }

    @IgnoreAuth
    @PostMapping("/cache/retention-query")
    @ControllerMethod("查询缓存的角色变动人员的保持率数据")
    fun queryCacheRoleChangeUser(@RequestBody roleChangeUserVO: @Valid RoleChangeUserVO?): Message<List<CustomerRetentionRatio>> {
        val list = customerRetentionRatioBizService.queryRoleChangeUserStash(roleChangeUserVO)
        return success(list)
    }

    @GetMapping("/prepare/send/changemq")
    @ControllerMethod("发送所有档案数据")
    fun prepareSendCustomerChangeMq(groupId: Long): Message<Void> {
        commonBizService.scyncAllCustomer(groupId)
        return success()
    }


    @GetMapping("/customer/rollback")
    @Authorization(AuthType.NONE)
    @ControllerMethod("回滚档案")
    fun customerRollback(
        vin: @NotBlank(message = "车架号不能为空") String?,
        @RequestParam(defaultValue = "2", required = false, name = "groupId") groupId: Long?
    ): Message<Void> {
        commonBizService.rollback(vin, groupId)
        return success()
    }

    @GetMapping("/send/msg")
    @ControllerMethod("测试新消息发送")
    fun sendMsg(): Message<Void> {
        hermesRpcService.sendVoiceMsg(
            "https://www.google.co.jp/images/branding/googlelogo/1x/googlelogo_color_272x92dp.png",
            797L
        )
        return success()
    }

    @PostMapping("/customer/inshop")
    @Authorization(AuthType.NONE)
    @ControllerMethod("模拟车辆进站")
    fun debugInShop(
        plateNo: @NotBlank(message = "车牌号号不能为空") String?,
        @RequestParam(defaultValue = "2", required = false, name = "groupId") groupId: Long?
    ): Message<Void> {
        plateNo?.let {
            followApiBizService.afterCarInShop(plateNo, groupId!!)
        }
        return success()
    }

    @GetMapping("/demo")
    @ControllerMethod("调用挂起函数测试方法")
    fun save(): Message<String> {
        val fet = CoroutineScope(Dispatchers.IO).future {
            async {
                // 耗时操作
                delay(3000)
                "Hello, Coroutines!"
            }.await()
        }
        val fet2 = CoroutineScope(Dispatchers.IO).future {
            async {
                // 耗时操作
                delay(5000)
                "Hello, Coroutines!"
            }.await()
        }

        fet2.join()
        val s2 = fet.join()
        return success(s2)
    }
}
