package cn.fw.valhalla.controller.task;

import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.common.utils.StringUtils;
import cn.fw.valhalla.domain.db.LeaveNeedDo;
import cn.fw.valhalla.domain.dto.RoleChangeDTO;
import cn.fw.valhalla.domain.enums.LeaveReasonEnum;
import cn.fw.valhalla.domain.enums.LeaveTodoTypeEnum;
import cn.fw.valhalla.service.bus.LeaveNeedDoBizService;
import cn.fw.valhalla.service.bus.pub.PubFollowBizService;
import cn.fw.valhalla.service.bus.pub.PubStandBizService;
import cn.fw.valhalla.service.report.CustomerRetentionRatioBizService;
import com.alibaba.fastjson.JSONObject;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;

/**
 * @author : kurisu
 * @className : CallReportDealTask
 * @description : 通话记录处理任务
 * @date: 2021-01-19 16:02
 */
@Slf4j
@Component
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class RoleChangeTask {
    private final LeaveNeedDoBizService leaveNeedDoBizService;
    private final StringRedisTemplate redisTemplate;
    private final PubFollowBizService pubFollowBizService;
    private final PubStandBizService pubStandBizService;
    private final CustomerRetentionRatioBizService customerRetentionRatioBizService;

    @Value("${spring.cache.custom.global-prefix}:mq:role")
    @Getter
    private String keyPrefix;

    @Value("${spring.cache.custom.global-prefix}:role-change:need-end-task")
    @Getter
    private String needDealTaskKey;


    @Value("${spring.cache.custom.global-prefix}:customer:api:distribution:afterthing")
    @Getter
    private String apiDistributionAfterThingKey;

    @Autowired
    public RoleChangeTask(final LeaveNeedDoBizService leaveNeedDoBizService,
                          final StringRedisTemplate redisTemplate,
                          final PubFollowBizService pubFollowBizService,
                          final PubStandBizService pubStandBizService,
                          final CustomerRetentionRatioBizService customerRetentionRatioBizService) {
        this.leaveNeedDoBizService = leaveNeedDoBizService;
        this.redisTemplate = redisTemplate;
        this.pubFollowBizService = pubFollowBizService;
        this.pubStandBizService = pubStandBizService;
        this.customerRetentionRatioBizService = customerRetentionRatioBizService;
    }


    /**
     * 服务顾问角色变动
     */
    @Scheduled(initialDelay = 1000 * 30, fixedRate = 1000 * 30)
    public void dealFwgwData() {
        List<String> failList = new ArrayList<>();
        String jsonStr;
        while ((jsonStr = redisTemplate.opsForList().leftPop(getRedisKey(RoleCode.FWGW))) != null) {
            final RoleChangeDTO roleChangeDTO = JSONObject.parseObject(jsonStr, RoleChangeDTO.class);
            if (Objects.isNull(roleChangeDTO)) {
                continue;
            }
            try {
                CompletableFuture.runAsync(() -> pubFollowBizService.onRoleChange(roleChangeDTO.getUserId(), roleChangeDTO.getShopId()));
                CompletableFuture.runAsync(() -> customerRetentionRatioBizService.roleChangeUserStash(roleChangeDTO.getUserId(), roleChangeDTO.getUserName(), roleChangeDTO.getShopId(), LocalDate.now()));
                pubStandBizService.removeStand(roleChangeDTO.getUserId(), roleChangeDTO.getShopId());
                leaveNeedDoBizService.addLeaveTodo(createDb(roleChangeDTO.getUserId(), roleChangeDTO.getShopId(), roleChangeDTO.getUserName()));
            } catch (Exception e) {
                if (StringUtils.isValid(jsonStr)) {
                    failList.add(jsonStr);
                }
                log.error("处理服务接待角色变动失败", e);
            }
        }
        if (!CollectionUtils.isEmpty(failList)) {
            redisTemplate.opsForList().rightPushAll(getRedisKey(RoleCode.FWGW), failList);
        }
    }

    /**
     * 续保角色
     */
    @Scheduled(initialDelay = 1000 * 30, fixedRate = 1000 * 30)
    public void dealXbData() {
        List<String> failList = new ArrayList<>();
        String jsonStr;
        while ((jsonStr = redisTemplate.opsForList().leftPop(getRedisKey(RoleCode.XBGJ))) != null) {
            RoleChangeDTO roleChangeDTO = JSONObject.parseObject(jsonStr, RoleChangeDTO.class);
            if (Objects.isNull(roleChangeDTO)) {
                continue;
            }
            try {
                leaveNeedDoBizService.xbgjChanged(roleChangeDTO);
            } catch (Exception e) {
                if (StringUtils.isValid(jsonStr)) {
                    failList.add(jsonStr);
                }
                log.error("处理续保跟进角色变动失败", e);
            }
        }
        if (!CollectionUtils.isEmpty(failList)) {
            redisTemplate.opsForList().rightPushAll(getRedisKey(RoleCode.XBGJ), failList);
        }
    }


    /**
     * 处理服务顾问角色变动后的跟进
     */
    @Scheduled(initialDelay = 1000 * 30, fixedRate = 1000 * 60)
    public void dealFwgwChangedTask() {
        List<String> failList = new ArrayList<>();
        String idStr;
        while ((idStr = redisTemplate.opsForList().leftPop(getNeedDealTaskKey())) != null) {
            if (!StringUtils.isNumber(idStr)) {
                continue;
            }
            try {
                Long leaveId = Long.valueOf(idStr);
                leaveNeedDoBizService.fwgwChanged(leaveId);
                leaveNeedDoBizService.checkAllcotaion(leaveId);
            } catch (Exception e) {
                if (StringUtils.isValid(idStr)) {
                    failList.add(idStr);
                }
                log.error("处理服务接待角色变动的跟进任务失败", e);
            }
        }
        if (!CollectionUtils.isEmpty(failList)) {
            redisTemplate.opsForList().rightPushAll(getNeedDealTaskKey(), failList);
        }
    }

    @Scheduled(initialDelay = 1000 * 15, fixedRate = 1000 * 30)
    public void apiDistributionAfter() {
        List<String> failList = new ArrayList<>();
        String datastr;
        while ((datastr = redisTemplate.opsForList().leftPop(getApiDistributionAfterThingKey())) != null) {
            if (StringUtils.isEmpty(datastr)) {
                continue;
            }
            try {
                leaveNeedDoBizService.apiDistributionAfter(datastr);
            } catch (Exception e) {
                if (StringUtils.isValid(datastr)) {
                    failList.add(datastr);
                }
                log.error("处理服务接待角色变动的跟进任务失败", e);
            }
        }
        if (!CollectionUtils.isEmpty(failList)) {
            redisTemplate.opsForList().rightPushAll(getApiDistributionAfterThingKey(), failList);
        }
    }


    private String getRedisKey(final String roleCode) {
        return String.format("%s:change:%s", getKeyPrefix(), roleCode);
    }

    private LeaveNeedDo createDb(Long userId, Long shopId, String userName) {
        LeaveNeedDo leaveNeedDo = new LeaveNeedDo();
        leaveNeedDo.setDone(Boolean.FALSE);
        leaveNeedDo.setEffectiveTime(new Date());
        leaveNeedDo.setReason(LeaveReasonEnum.CHANGE);
        leaveNeedDo.setType(LeaveTodoTypeEnum.CUSTOMER);
        leaveNeedDo.setShopId(shopId);
        leaveNeedDo.setUserId(userId);
        leaveNeedDo.setUserName(userName);
        leaveNeedDo.setCreateTime(new Date());
        return leaveNeedDo;
    }
}
