package cn.fw.valhalla.service.bus.cust;

import cn.fw.common.constant.CommonConstant;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.pool.StammkundePool;
import cn.fw.valhalla.domain.dto.StammkundeDto;
import cn.fw.valhalla.domain.enums.DefeatReasonEnum;
import cn.fw.valhalla.domain.enums.StammkundeSourcesEnum;
import cn.fw.valhalla.domain.enums.StammkundeStatusEnum;
import cn.fw.valhalla.rpc.ehr.EhrRpcService;
import cn.fw.valhalla.rpc.erp.UserService;
import cn.fw.valhalla.rpc.erp.dto.PostUserDTO;
import cn.fw.valhalla.rpc.erp.dto.UserInfoDTO;
import cn.fw.valhalla.rpc.oop.OopService;
import cn.fw.valhalla.rpc.oop.dto.ShopDTO;
import cn.fw.valhalla.service.data.CustomerService;
import cn.fw.valhalla.service.data.StammkundePoolService;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Objects;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * 档案api业务服务
 *
 * @author : kurisu
 * @version : 1.3.1
 * @className : CustomerApiBizService
 * @description : 档案api业务服务
 * @date : 2023-06-15 15:08
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class CustomerApiBizService {
    private final static String TAG = "CustomerApiBizService";
    private final CustomerService customerService;
    private final OopService oopService;
    private final UserService userService;
    private final EhrRpcService ehrRpcService;
    private final StammkundePoolService stammkundePoolService;
    private final StringRedisTemplate stringRedisTemplate;

    @Value("${spring.cache.custom.global-prefix}:customer:api:distribution:lock")
    @Getter
    private String keyPrefix;

    @Value("${spring.cache.custom.global-prefix}:customer:api:distribution:afterthing")
    @Getter
    private String afterThingKey;

    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getKeyPrefix()", key = "#vin", message = "请勿重复调用")
    public void distribution4api(final String vin, final Long staffId, final Long shopId) {
        final ShopDTO shop = oopService.shop(shopId);
        BV.notNull(shop, () -> "shop does not exist");
        UserInfoDTO user = ehrRpcService.user(staffId);
        BV.notNull(user, () -> "user does not exist");
        final Customer customer = customerService.queryByFrameNo(vin, shop.getGroupId());
        if (customer == null) {
            log.error("*{}* customer[{}] does not exist", TAG, vin);
            return;
        }
        boolean distributable = distributable(customer, shop, user);
        if (!distributable) {
            return;
        }
        final Long adviserId = customer.getAdviserId();
        doDistribution(customer, shop, user);
        afterDistribution(vin, shop.getGroupId(), adviserId);
    }

    private boolean distributable(final Customer customer, final ShopDTO shop, final UserInfoDTO staff) {
        final Long originShopId = customer.getShopId();
        final Long adviserId = customer.getAdviserId();
        if (originShopId == null) {
            log.error("*{}* customer in public pool", TAG);
            return false;
        }
        if (Objects.equals(staff.getId(), adviserId)) {
            log.error("*{}* staff is same as origin", TAG);
            return false;
        }
        List<PostUserDTO> rolesUser = userService.getUserByRole(shop.getId(), RoleCode.FWGW);
        boolean isMatch = rolesUser.stream().anyMatch(user -> user.getUserId().equals(staff.getId()));
        if (!isMatch) {
            log.error("*{}* staff don't has role", TAG);
            return false;
        }
        List<PostUserDTO> originRolesUser = userService.getUserByRole(originShopId, RoleCode.FWGW);
        boolean match = originRolesUser.stream().anyMatch(user -> user.getUserId().equals(adviserId));
        if (match) {
            log.error("*{}* origin staff still have role", TAG);
            return false;
        }
        return true;
    }

    private void doDistribution(final Customer customer, final ShopDTO shop, final UserInfoDTO staff) {
        customer.setShopId(shop.getId());
        customer.setAdviserId(staff.getId());
        rejectPool(customer.getId(), shop, staff);
        StammkundePool stammkundePool = createPool(customer, staff);
        customerService.updateById(customer);
        stammkundePoolService.save(stammkundePool);
    }

    private void rejectPool(Long customerId, ShopDTO shop, UserInfoDTO staff) {
        StammkundeDto stammkundeDto = new StammkundeDto();
        stammkundeDto.setCustomerId(customerId);
        stammkundeDto.setReason(DefeatReasonEnum.LC);
        stammkundeDto.setNewUserId(staff.getId());
        stammkundeDto.setNewUserName(staff.getUserName());
        stammkundeDto.setNewShopId(shop.getId());
        stammkundeDto.setNewShopName(shop.getShortName());
        stammkundeDto.setGroupId(shop.getGroupId());
        stammkundePoolService.reject(stammkundeDto);
    }

    private StammkundePool createPool(Customer customer, UserInfoDTO staff) {
        StammkundePool sp = new StammkundePool();
        sp.setCustomerId(customer.getId());
        sp.setAdviserId(customer.getAdviserId());
        sp.setAdviserName(staff.getUserName());
        sp.setSources(StammkundeSourcesEnum.SYSTEMATIC);
        sp.setPoolStatus(StammkundeStatusEnum.KUNDE);
        sp.setAktiv(Boolean.TRUE);
        sp.setActivationTime(new Date());
        sp.setShopId(customer.getShopId());
        sp.setGroupId(customer.getGroupId());
        return sp;
    }

    private void afterDistribution(String vin, Long groupId, Long originUserId) {
        String data = vin + CommonConstant.Symbol.PER_CENT + groupId + CommonConstant.Symbol.PER_CENT + originUserId;
        BoundListOperations<String, String> listOps = stringRedisTemplate.boundListOps(getAfterThingKey());
        listOps.rightPush(data);
    }
}
