package cn.fw.valhalla.service.bus.follow;

import cn.fw.common.constant.CommonConstant;
import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.domain.db.OriginalData;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.follow.ClueTask;
import cn.fw.valhalla.domain.db.follow.FollowClue;
import cn.fw.valhalla.domain.enums.ClueStatusEnum;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.vo.setting.SettingVO;
import cn.fw.valhalla.sdk.enums.DataTypeEnum;
import cn.fw.valhalla.service.bus.follow.strategy.FollowStrategy;
import cn.fw.valhalla.service.data.ClueTaskService;
import cn.fw.valhalla.service.data.CustomerService;
import cn.fw.valhalla.service.data.FollowClueService;
import cn.fw.valhalla.service.event.TaskCancelEvent;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author : kurisu
 * @className : FollowBizService
 * @description : 跟进业务服务
 * @date: 2020-08-14 17:27
 */
@Slf4j
@Service
public class FollowBizService {
    /**
     * 跟进处理器map
     */
    private final Map<FollowTypeEnum, FollowStrategy> followMap;
    private final CustomerService customerService;
    private final FollowClueService followClueService;
    private final ClueTaskService clueTaskService;

    @Value("${follow.flowNo}")
    @Getter
    private String flowNo;

    @Value("${spring.cache.custom.global-prefix}:follow:defeat")
    @Getter
    private String keyPrefix;

    @Value("${spring.cache.custom.global-prefix}:follow:rpc:complete")
    @Getter
    private String completeKeyPrefix;

    @Autowired
    public FollowBizService(final List<FollowStrategy> followStrategyList,
                            final CustomerService customerService,
                            final FollowClueService followClueService,
                            final ClueTaskService clueTaskService) {
        this.followMap = followStrategyList.stream().collect(Collectors.toMap(FollowStrategy::getFollowType, v -> v));
        this.customerService = customerService;
        this.followClueService = followClueService;
        this.clueTaskService = clueTaskService;
    }

    /**
     * 更新跟进任务
     *
     * @param setting
     * @param type
     * @param groupId
     */
    public void synFollowTask(List<SettingVO> setting, FollowTypeEnum type, Long groupId) {
        FollowStrategy strategy = followMap.get(type);
        Assert.notNull(strategy, "strategy cannot be null");
        strategy.settingChanged(setting, groupId);
    }

    /**
     * 当业务数据取消的时候
     *
     * @param event
     */
    public void onDataCancel(TaskCancelEvent event) {
        FollowTypeEnum typeEnum = conv(event.getDataTypeEnum());
        if (Objects.isNull(typeEnum) || FollowTypeEnum.AC.equals(typeEnum)) {
            return;
        }
        Long customerId = event.getCustomerId();
        Long groupId = event.getGroupId();
        Customer customer = customerService.queryById(customerId);
        if (Objects.isNull(customer)) {
            return;
        }
        FollowClue followClue = followClueService.getOne(Wrappers.<FollowClue>lambdaQuery()
                        .eq(FollowClue::getVin, customer.getFrameNo())
                        .eq(FollowClue::getGroupId, groupId)
                        .eq(FollowClue::getClueType, typeEnum)
                        .eq(FollowClue::getClueState, ClueStatusEnum.WAITING)
                , Boolean.FALSE);
        if (Objects.isNull(followClue)) {
            return;
        }
        followClueService.removeById(followClue.getId());
    }

    /**
     * 元数据生成任务
     *
     * @param originalData
     * @param typeEnum
     */
    public boolean origin2task(OriginalData originalData, DataTypeEnum typeEnum) {
        if (Objects.isNull(originalData)) {
            return true;
        }
        FollowTypeEnum type = conv(typeEnum);
        FollowStrategy strategy = followMap.get(type);
        Assert.notNull(strategy, "strategy cannot be null");
        try {
            return strategy.origin2task(originalData);
        } catch (Exception exception) {
            log.error("处理元数据失败 data: [{}]", originalData, exception);
            return false;
        }
    }

    /**
     * 开始线索
     *
     * @param cluePool
     */
    public void startClue(FollowClue cluePool) {
        try {
            FollowStrategy strategy = followMap.get(cluePool.getClueType());
            Assert.notNull(strategy, "strategy cannot be null");

            strategy.startClue(cluePool);
        } catch (Exception e) {
            log.error("开始线索失败 data: [{}]", cluePool, e);
        }
    }

    /**
     * 定时任务结束任务
     *
     * @param task
     */
    public void endTask(ClueTask task) {
        FollowStrategy strategy = followMap.get(task.getType());
        Assert.notNull(strategy, "strategy cannot be null");
        try {
            strategy.closeTask(task);
        } catch (Exception ex) {
            log.error("定时任务结束任务失败 taskId: [{}]", task.getId(), ex);
        }
    }

    /**
     * 角色变动结束任务
     *
     * @param task
     */
    public void roleChangeEndTask(ClueTask task) {
        FollowStrategy strategy = followMap.get(task.getType());
        Assert.notNull(strategy, "strategy cannot be null");
        strategy.onRoleChangeCloseTask(task);
    }

    /**
     * 任务结束同步状态到跟进系统
     *
     * @param task
     */
    public void syncEndTask(ClueTask task) {
        FollowStrategy strategy = followMap.get(task.getType());
        Assert.notNull(strategy, "strategy cannot be null");
        try {
            strategy.syncEndTask(task);
        } catch (Exception ex) {
            log.error("任务结束同步状态到跟进系统失败 taskId: [{}]", task.getId(), ex);
        }
    }

    public void checkInsTask(String str) {
        FollowStrategy strategy = followMap.get(FollowTypeEnum.IR);
        Assert.notNull(strategy, "strategy cannot be null");
        try {
            String[] dataArr = str.split(CommonConstant.Symbol.AT);
            if (dataArr.length < 4) {
                log.info("检查续保任务是否需要终止失败：{}", str);
                return;
            }
            String vin = dataArr[0];
            Long adviserId = Long.parseLong(dataArr[1]);
            Long groupId = Long.parseLong(dataArr[2]);
            strategy.checkInsTask(vin, adviserId, groupId);
        } catch (Exception ex) {
            log.error("检查续保任务是否需要终止失败: [{}]", str);
        }
    }

    /**
     * 跟进完成
     *
     * @param clue          线索
     * @param overdue       逾期
     * @param callEfficient 有效电话
     */
    public void afterFollowComplete(FollowClue clue, boolean overdue, boolean callEfficient) {
        FollowStrategy strategy = followMap.get(clue.getClueType());
        Assert.notNull(strategy, "strategy cannot be null");
        strategy.onFollowComplete(clue, overdue, callEfficient);
    }

    /**
     * 档案进入公共池任务战败
     *
     * @param vin
     * @param groupId
     */
    @Transactional(rollbackFor = Exception.class)
    public void stopClue(String vin, Long groupId, boolean fromTask) {
        List<FollowTypeEnum> typeList = Arrays.asList(FollowTypeEnum.FM, FollowTypeEnum.RM);

        List<FollowClue> cluePoolList = followClueService.list(Wrappers.<FollowClue>lambdaQuery()
                .eq(FollowClue::getClueState, ClueStatusEnum.ONGOING)
                .in(FollowClue::getClueType, typeList)
                .eq(FollowClue::getVin, vin)
                .eq(FollowClue::getGroupId, groupId)
        );
        if (!CollectionUtils.isEmpty(cluePoolList)) {
            for (FollowClue clue : cluePoolList) {
                FollowStrategy strategy = followMap.get(clue.getClueType());
                if (Objects.nonNull(strategy)) {
                    strategy.forceStopClue(clue, fromTask);
                }
            }
        }
    }

    /**
     * 终止任务
     *
     * @param task
     */
    @Transactional(rollbackFor = Exception.class)
    public void stopTask(ClueTask task) {
        FollowStrategy strategy = followMap.get(task.getType());
        if (Objects.nonNull(strategy)) {
            strategy.forceStopTask(task);
        }
    }

    public void stopClueTodoForInShop(Long clueId) {
        FollowClue clue = followClueService.getById(clueId);
        if (Objects.isNull(clue) || !Boolean.TRUE.equals(clue.getInShop())) {
            return;
        }
        FollowStrategy strategy = followMap.get(clue.getClueType());
        if (Objects.nonNull(strategy)) {
            strategy.stopClueTodoForInShop(clue);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void doSecondFollow(Long taskId) {
        ClueTask task = clueTaskService.getById(taskId);
        if (Objects.isNull(task)) {
            return;
        }
        FollowStrategy strategy = followMap.get(task.getType());
        if (Objects.nonNull(strategy)) {
            strategy.doSecondFollow(task);
        }
    }


    /**
     * 手动开始任务
     */
    public void manualStartClue() {
        List<FollowClue> list = followClueService.list(Wrappers.<FollowClue>lambdaQuery()
                .eq(FollowClue::getClueState, ClueStatusEnum.WAITING)
                .le(FollowClue::getStartTime, DateUtil.localDateTime2Date(LocalDateTime.now()))
                .last("limit 0, 2000")
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        for (FollowClue cluePool : list) {
            startClue(cluePool);
        }
    }


    private FollowTypeEnum conv(DataTypeEnum type) {
        switch (type) {
            case AS:
                return FollowTypeEnum.AC;
            case OD:
            case FM:
                return FollowTypeEnum.FM;
            case BI:
                return FollowTypeEnum.IR;
            case FS:
                return FollowTypeEnum.RM;
            default:
                return null;
        }
    }

}
